/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.DataExchange;

import java.util.HashMap;
import java.util.Properties;
import java.util.UUID;

import org.apache.log4j.Logger;

import sos.net.ssh.SOSSSHJob2;
import sos.net.ssh.SOSSSHJobOptions;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.io.Files.JSFile;
import com.sos.VirtualFileSystem.DataElements.SOSFileList;
import com.sos.VirtualFileSystem.DataElements.SOSFileListEntry;
import com.sos.VirtualFileSystem.FTP.SOSFTPOptions;
import com.sos.VirtualFileSystem.Factory.VFSFactory;
import com.sos.VirtualFileSystem.Interfaces.ISOSVFSHandler;
import com.sos.VirtualFileSystem.Options.SOSConnection2Options;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsAlternate;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
* \class Jade4DMZ 
* 
* \brief Jade4DMZ - 
* 
* \details
*
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author KB
* \version $Id$
* \see reference
*
* Created on 06.06.2012 16:15:21
 */

/**
 * @author KB
 *
 */
@I18NResourceBundle(baseName = "SOSDataExchange", defaultLocale = "en")
public class Jade4DMZ extends JSToolBox implements Runnable {

	@SuppressWarnings("unused")
	private final String		conClassName				= "Jade4DMZ";
	private static final String	conSVNVersion				= "$Id$";
	private static final Logger	logger						= Logger.getLogger(Jade4DMZ.class);

	private SOSFTPOptions		objOptions					= null;
	private SOSFileList			objSourceFileList			= null;

	private SOSSSHJob2			objM						= null;
	private ISOSVFSHandler		objVFS						= null;
	protected SOSSSHJobOptions	objO						= null;
	protected String			strTempFolderNameOnDMZ		= null;

	private boolean				CreateTempFolderOnDMZ		= false;
	private boolean				StartTransferToDMZ			= false;
	private boolean				StartTransferFromDMZ2Inet	= false;
	private boolean				StartTransferFromDMZ2Intranet	= false;
	private boolean				RemoveFilesOnDMZ			= false;
	private boolean				RemoveTempFolderOnDMZ		= false;
	private boolean				RemoveFilesOnIntranet		= false;
	private boolean RemoveFilesOnInet = false;
	
	private SOSFileList lstFilesTransferredFromIntranet = null;
	private long lngNoOfFilesTransferredFromIntranet = -1;
	private SOSFileList lstFilesTransferredFromDMZ2Intranet = null;
	private long lngNoOfFilesTransferredFromDMZ2Intranet = -1;
	
	public Jade4DMZ() {
		super("SOSDataExchange");
		init();
		//
	}

	/**
	 * @param settingsFile
	 * @param settingsSection
	 * @param logger
	 * @param arguments_
	 */
	public Jade4DMZ(Properties pobjProperties) throws Exception {
		this();
		this.Options();
		// TODO Properties in die OptionsClasse weiterreichen
		// objOptions.setAllOptions(pobjProperties);
	}

	public Jade4DMZ(SOSFTPOptions pobjOptions) throws Exception {
		this();
		this.objOptions = pobjOptions;
	}

	public Jade4DMZ(HashMap<String, String> pobjJSSettings) throws Exception {
		this();
		this.Options().setAllOptions(pobjJSSettings);
	}

	public SOSFTPOptions Options() {
		if (objOptions == null) {
			objOptions = new SOSFTPOptions();
		}
		return objOptions;
	}

	public SOSSSHJobOptions SSHOptions() {
		if (objO == null) {
			objO = objM.Options();
		}
		return objO;
	}

	private void init() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init";
		
		// TODO ber Option steuern
		/**
		 * Trilead ist leider nicht mulithreading-faehig und ausserdem extrem langsam
		 * JCraft kann threading ohne probleme
		 */
		VFSFactory.sFTPHandlerClassName = "com.sos.VirtualFileSystem.SFTP.SOSVfsSFtpJCraft";
		
		logger.info(conClassName + " --- " + conSVNVersion);
		objM = new SOSSSHJob2();
		objM.keepConnected = true;
		objO = objM.Options();

		this.Options();

	} // private void init

	@Override
	public void run() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Run";
		try {
			this.Execute();
		}
		catch (Exception e) {
			throw new JobSchedulerException("abort", e);
		}
	}

	public boolean Execute() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::execute";
		boolean flgOK = true;

		objOptions.getTextProperties().put("version", conSVNVersion);

		CheckJumpSettings();
		EstablishSSHConnection();

		try {
			CreateTempFolderOnDMZ();

			if (objOptions.operation.isOperationSendUsingDMZ()) { // Intranet via DMZ to Internet
				try {
					StartTransferFromIntranet2DMZ();
					StartTransferFromDMZ2Inet();
					RemoveFilesOnIntranet();
				}
				catch (Exception e) {
					e.printStackTrace();
					throw e;
				}
			}
			else { // Internet via DMZ to Internet
				if (objOptions.operation.isOperationReceiveUsingDMZ()) {
					StartTransferFromInet2DMZ();
					StartTransferFromDMZ2Intranet();
					RemoveFilesFromINet();
				}
				else {
					throw new JobSchedulerException(Messages.getMsg("Jade4DMZ-E-001"));
					// "jump host" / DMZ nur mit isOperationSendUsingDMZ oder isOperationReceiveUsingDMZ mglich
				}
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			flgOK = false;
		}
		finally {
			RemoveFilesOnDMZ();
			RemoveTempFolderOnDMZ();
			objM.DisConnect();
		}
		return flgOK;
	}

	private void RemoveFilesFromINet() {
		
		if (objOptions.remove_files.value() == true) {
//			JSFile fleFileList = new JSFile.get;
//			lstFilesTransferredFromDMZ2Intranet.Write2File();
			/**
			 * 1) File bertragen ins temp-Verzeichnis
			 * 2) starten client auf dmz mit delete-option und parameter filelist=
			 */
			
		}
		RemoveFilesOnInet = true;
	}

	private void StartTransferFromDMZ2Intranet() {
		// TODO Auto-generated method stub

		SOSFTPOptions obj2DMZ = new SOSFTPOptions();

		String strB = objOptions.getOptionsAsCommandLine();

		SOSConnection2OptionsAlternate objSource = objOptions.Source();
		String strD = objSource.getOptionsAsCommandLine();
		strD = strD.replaceAll("source_", "");
		obj2DMZ.CommandLineArgs(strB);
		
		obj2DMZ.remote_dir.Value(strTempFolderNameOnDMZ);

		obj2DMZ.remove_files.value(false);

		obj2DMZ.settings.Value(null);
		obj2DMZ.profile.Value(null);
		
		obj2DMZ.user.Value(objOptions.jump_user.Value());
		obj2DMZ.password.Value(objOptions.jump_password.Value());
		obj2DMZ.protocol.Value(objOptions.jump_protocol.Value());
		obj2DMZ.host.Value(objOptions.jump_host.Value());
		obj2DMZ.auth_method.Value(objOptions.jump_ssh_auth_method.Value());
		
		obj2DMZ.local_dir.Value(objOptions.TargetDir.Value());

		try {
			JadeEngine objJade = new JadeEngine(obj2DMZ);
			objJade.Execute();
			lstFilesTransferredFromIntranet = objJade.getFileList();
//			lngNoOfFilesTransferredFromIntranet = lstFilesTransferredFromIntranet.count();
			StartTransferFromDMZ2Intranet = true;
		}
		catch (Exception e) {
			throw new JobSchedulerException("Transfer failed", e);
		}

	}

	private void StartTransferFromInet2DMZ() {

		SOSFTPOptions obj2DMZ = objOptions.getClone();

		SOSConnection2OptionsAlternate objSource = objOptions.Source();
		String strD = objSource.getOptionsAsCommandLine();
		strD = strD.replaceAll("source_", "");
		obj2DMZ.CommandLineArgs(strD);
		
		setDMZasTarget4Receive(obj2DMZ);

		obj2DMZ.settings.Value(null);
		obj2DMZ.profile.Value(null);
		
		obj2DMZ.remote_dir.Value(objOptions.SourceDir.Value());
		obj2DMZ.host.Value(objSource.host.Value());
		String strC = obj2DMZ.getOptionsAsCommandLine();

		strC += " -log_filename=" + getUniqueFileName("log");
		strC += " -createResultSet=true -ResultSetFileName=" + getUniqueFileName("fls");
		executeSSHCommand(objOptions.jump_command.Value() + " " + strC + " " + strD);
	}

	private void CreateTempFolderOnDMZ() {

		final String conMethodName = conClassName + "::CreateTempFolderOnDMZ";
		logger.trace(conMethodName);

		try {
			strTempFolderNameOnDMZ = getUniqueFileName("");
			executeSSHCommand("mkdir " + strTempFolderNameOnDMZ);
		}
		catch (Exception e) {
			throw new JobSchedulerException("transfer failed", e);
		}
		CreateTempFolderOnDMZ = true;
	} // private void CreateTempFolderOnDMZ

	private String getUniqueFileName(final String pstrFileNameExtension) {
		
		@SuppressWarnings("unused")
		final String	conMethodName	= conClassName + "::getUniqueFileName";
		
		String strUUID = UUID.randomUUID().toString();
		String strE = "";
		if (pstrFileNameExtension.length() > 0) {
			strE = "." + pstrFileNameExtension;
		}
		String strLogFileName = "/tmp/jade-" + strUUID + strE;
		
		return strLogFileName;
	} // private String getUniqueFileName
	
	private void StartTransferFromIntranet2DMZ() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::StartTransferToDMZ";
		logger.trace(conMethodName);

		SOSFTPOptions obj2DMZ = objOptions.getClone();
		setDMZasTarget(obj2DMZ);

		try {
			JadeEngine objJade = new JadeEngine(obj2DMZ);
			objJade.Execute();
			lstFilesTransferredFromIntranet = objJade.getFileList();
			lngNoOfFilesTransferredFromIntranet = lstFilesTransferredFromIntranet.count();
			StartTransferToDMZ = true;
		}
		catch (Exception e) {
			throw new JobSchedulerException("Transfer failed", e);
		}
	} // private void StartTransferToDMZ

	private void setDMZasTarget4Receive(SOSFTPOptions objDMZOptions) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setDMZasTarget";

		objDMZOptions.local_dir.Value(strTempFolderNameOnDMZ);

		objDMZOptions.transactional.value(true);
		objDMZOptions.remove_files.value(false);   // delete on source platform at the end of the overall transfer

		objDMZOptions.replacement.Value(null);
		objDMZOptions.replacing.Value(null);
		objDMZOptions.atomic_prefix.Value("");
		objDMZOptions.atomic_suffix.Value("");

		objDMZOptions.ClearJumpParameter();
		
		objDMZOptions.remote_dir.Value(objDMZOptions.TargetDir.Value());
	} // private void setDMZasTarget

	private void setDMZasTarget(SOSFTPOptions objDMZOptions) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setDMZasTarget";

		objDMZOptions.remote_dir.Value(strTempFolderNameOnDMZ);
		objDMZOptions.user.Value(objOptions.jump_user.Value());
		objDMZOptions.password.Value(objOptions.jump_password.Value());
		objDMZOptions.protocol.Value(objOptions.jump_protocol.Value());
		objDMZOptions.host.Value(objOptions.jump_host.Value());
		objDMZOptions.auth_method.Value(objOptions.jump_ssh_auth_method.Value());

		objDMZOptions.transactional.value(true);
		objDMZOptions.remove_files.value(false);   // delete on source platform at the end of the overall transfer

		objDMZOptions.replacement.Value(null);
		objDMZOptions.replacing.Value(null);
		objDMZOptions.atomic_prefix.Value("");
		objDMZOptions.atomic_suffix.Value("");

	} // private void setDMZasTarget

	private void setDMZasSource(SOSFTPOptions objDMZOptions) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setDMZasTarget";

		objDMZOptions.ClearJumpParameter();
		objDMZOptions.local_dir.Value(strTempFolderNameOnDMZ);

		objDMZOptions.remove_files.value(false);

	} // private void setDMZasTarget

	private void StartTransferFromDMZ2Inet() {

		final String conMethodName = conClassName + "::StartTransferFromDMZ2Inet";
		logger.trace(conMethodName);

		String strB = objOptions.getOptionsAsCommandLine();
		SOSFTPOptions obj2Inet = new SOSFTPOptions();
		try {
			SOSConnection2OptionsAlternate objTarget = objOptions.Target();
			String strD = objTarget.getOptionsAsCommandLine();
			obj2Inet.CommandLineArgs(strB);
			obj2Inet.settings.Value(null);
			obj2Inet.profile.Value(null);

			obj2Inet.host.Value(objTarget.host.Value());
			if (obj2Inet.TargetDir.isDirty()) {
				obj2Inet.remote_dir.Value(obj2Inet.TargetDir.Value());
			}
			setDMZasSource(obj2Inet);
			String strE = strD.replaceAll("target_", "");
			String strC = obj2Inet.getOptionsAsCommandLine();

			String strUUID = UUID.randomUUID().toString();
			String strLogFileName = "/tmp/jade-" + strUUID + ".log";
			strE += " -log_filename=" + strLogFileName;
			executeSSHCommand(objOptions.jump_command.Value() + " " + strC + " " + strD + " " + strE);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	} // private void StartTransferFromDMZ2Inet

	private void RemoveFilesOnDMZ() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::RemoveFilesOnDMZ";
		logger.trace(conMethodName);

		if (StartTransferToDMZ == true) {
			// remove Files
		}
	} // private void RemoveFilesOnDMZ

	private void RemoveTempFolderOnDMZ() {

		final String conMethodName = conClassName + "::RemoveTempFolderOnDMZ";
		logger.trace(conMethodName);
		if (CreateTempFolderOnDMZ == true) {
			executeSSHCommand("rm -f -R " + strTempFolderNameOnDMZ);
			CreateTempFolderOnDMZ = false;
			RemoveTempFolderOnDMZ = true;
		}
	} // private void RemoveTempFolderOnDMZ

	private void executeSSHCommand(final String pstrCommand) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::executeCommand";

		try {
			logger.debug(pstrCommand);
			objO.command.Value(pstrCommand);
			objM.Execute();

			String strStdOut = objM.getStdOut().toString();
			String strStdErr = objM.getStdErr().toString();
			objM.Clear();
		}
		catch (Exception e) {
			throw new JobSchedulerException("Transfer failed", e);
		}

	} // private void executeCommand

	private void RemoveFilesOnIntranet() {

		final String conMethodName = conClassName + "::RemoveFilesFromIntranet";
		logger.debug(conMethodName);

		if (objOptions.remove_files.value() == true) {
			for (SOSFileListEntry objFile : lstFilesTransferredFromIntranet.List()) {
				if (objFile.FileExists() == true) {
					objFile.DeleteSourceFile();
				}
			}
		}
		RemoveFilesOnIntranet = true;
	} // private void RemoveFilesFromIntranet

	private void CheckJumpSettings() {

		final String conMethodName = conClassName + "::CheckJumpSettings";
		logger.trace(conMethodName);

	} // private void CheckJumpSettings

	private void EstablishSSHConnection() {

		final String conMethodName = conClassName + "::EstablishSSHConnection";

		try {
			objO.auth_file = objOptions.getjump_ssh_auth_file();
			objO.auth_method = objOptions.getjump_ssh_auth_method();
			objO.host = objOptions.getjump_host();
			objO.user = objOptions.getjump_user();
			objO.password = objOptions.getjump_password();
			objO.port = objOptions.getjump_port();

			logger.info(objO.toString());

			// TODO: connect as method in SSHJob2

			objM.Connect();
		}
		catch (Exception e) {
			logger.error(conMethodName + ": " + "Error occured ..." + e.getMessage());
			throw new JobSchedulerException("Transfer failed", e);
		}
	} // private void EstablishSSHConnection
}
