/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.creator.jobs;

import java.util.HashMap;

import com.sos.JSHelper.Options.*;
import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener; 
import org.apache.log4j.Logger;

import com.sos.hibernate.options.HibernateOptions;

/**
 * \class 		JobNetCreatorOptionsSuperClass - creates the job net for a given bootstrap order
 *
 * \brief 
 * An Options-Super-Class with all Options. This Class will be extended by the "real" Options-class (\see JobNetCreatorOptions.
 * The "real" Option class will hold all the things, which are normaly overwritten at a new generation
 * of the super-class.
 *
 *

 *
 * see \see C:\Users\schaedi\AppData\Local\Temp\scheduler_editor-3377211190692187569.html for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by C:\ProgramData\sos-berlin.com\jobscheduler\scheduler-1.3.12.2032\config\JOETemplates\java\xsl\JSJobDoc2JSOptionSuperClass.xsl from http://www.sos-berlin.com at 20120228153423 
 * \endverbatim
 * \section OptionsTable Tabelle der vorhandenen Optionen
 * 
 * Tabelle mit allen Optionen
 * 
 * MethodName
 * Title
 * Setting
 * Description
 * IsMandatory
 * DataType
 * InitialValue
 * TestValue
 * 
 * 
 *
 * \section TestData Eine Hilfe zum Erzeugen einer HashMap mit Testdaten
 *
 * Die folgenden Methode kann verwendet werden, um fr einen Test eine HashMap
 * mit sinnvollen Werten fr die einzelnen Optionen zu erzeugen.
 *
 * \verbatim
 private HashMap <String, String> SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM) {
	pobjHM.put ("		JobNetCreatorOptionsSuperClass.auth_file", "test");  // This parameter specifies the path and name of a user's pr
		return pobjHM;
  }  //  private void SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM)
 * \endverbatim
 */
@JSOptionClass(name = "JobNetCreatorOptionsSuperClass", description = "JobNetCreatorOptionsSuperClass")
public class JobNetCreatorOptionsSuperClass extends HibernateOptions {

	private static final long serialVersionUID = 2632562825510369271L;
	private final String					conClassName						= "JobNetCreatorOptionsSuperClass";
		@SuppressWarnings("unused")
	private static Logger		logger			= Logger.getLogger(JobNetCreatorOptionsSuperClass.class);
		
	@SuppressWarnings("unused")
	private	final String		conSVNVersion			= "$Id: JobNetCreatorOptionsSuperClass.java 16850 2012-03-21 12:14:23Z ss $";

/**
 * \var uuid_jobnet_identifier : 
 * 
 *
 */
    @JSOptionDefinition(name = "uuid_jobnet_identifier", 
    description = "", 
    key = "uuid_jobnet_identifier", 
    type = "SOSOptionUUID", 
    mandatory = false)
    
    public SOSOptionUUID uuid_jobnet_identifier = new SOSOptionUUID(this, conClassName + ".uuid_jobnet_identifier", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getuuid_jobnet_identifier : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionUUID  getuuid_jobnet_identifier() {
        return uuid_jobnet_identifier;
    }

/**
 * \brief setuuid_jobnet_identifier : 
 * 
 * \details
 * 
 *
 * @param uuid_jobnet_identifier : 
 */
    public void setuuid_jobnet_identifier (SOSOptionUUID p_uuid_jobnet_identifier) { 
        this.uuid_jobnet_identifier = p_uuid_jobnet_identifier;
    }

/**
 * \var JobChainName : 
 * 
 *
 */
    @JSOptionDefinition(name = "JobChainName", 
    description = "", 
    key = "JobChainName", 
    type = "SOSOptionString", 
    mandatory = true)
    
    public SOSOptionString JobChainName = new SOSOptionString(this, conClassName + ".JobChainName", // HashMap-Key
                                                                "", // Titel
                                                                " ", // InitValue
                                                                " ", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getJobChainName : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionString  getJobChainName() {
        return JobChainName;
    }

/**
 * \brief setJobChainName : 
 * 
 * \details
 * 
 *
 * @param JobChainName : 
 */
    public void setJobChainName (SOSOptionString p_JobChainName) { 
        this.JobChainName = p_JobChainName;
    }

                        

/**
 * \var OrderId : 
 * 
 *
 */
    @JSOptionDefinition(name = "OrderId", 
    description = "", 
    key = "OrderId", 
    type = "SOSOptionString", 
    mandatory = true)
    
    public SOSOptionString OrderId = new SOSOptionString(this, conClassName + ".OrderId", // HashMap-Key
                                                                "", // Titel
                                                                " ", // InitValue
                                                                " ", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getOrderId : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionString  getOrderId() {
        return OrderId;
    }

/**
 * \brief setOrderId : 
 * 
 * \details
 * 
 *
 * @param OrderId : 
 */
    public void setOrderId (SOSOptionString p_OrderId) { 
        this.OrderId = p_OrderId;
    }

                        

/**
 * \var host : 
 * 
 *
 */
    @JSOptionDefinition(name = "SCHEDULER_HOST", 
    description = "", 
    key = "SCHEDULER_HOST", 
    type = "SOSOptionHostName", 
    mandatory = true)
    
    public SOSOptionHostName SCHEDULER_HOST = new SOSOptionHostName(this, conClassName + ".SCHEDULER_HOST", // HashMap-Key
                                                                "", // Titel
                                                                " ", // InitValue
                                                                " ", // DefaultValue
                                                                true // isMandatory
                    );
    
//    public SOSOptionHostName hostName = (SOSOptionHostName) SCHEDULER_HOST.SetAlias("");

/**
 * \brief gethost : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionHostName  getSCHEDULER_HOST() {
        return SCHEDULER_HOST;
    }

/**
 * \brief sethost : 
 * 
 * \details
 * 
 *
 * @param host : 
 */
    public void setSCHEDULER_HOST (SOSOptionHostName p_host) { 
        this.SCHEDULER_HOST = p_host;
    }

                        

/**
 * \var jobnet : 
 * Name of the job net
 *
 */
    
//    @JSOptionDefinition(name = "JobNet", 
//    description = "", 
//    key = "JobNet", 
//    type = "SOSOptionString", 
//    mandatory = true)
//    
//    public SOSOptionString JobNet = new SOSOptionString(this, conClassName + ".JobNet", // HashMap-Key
//                                                                "", // Titel
//                                                                " ", // InitValue
//                                                                " ", // DefaultValue
//                                                                true // isMandatory
//                    );

/**
 * \brief getjobnet : 
 * 
 * \details
 * Name of the job net
 *
 * \return 
 *
 */
//    public SOSOptionString  getJobNet() {
//        return JobNet;
//    }

/**
 * \brief setjobnet : 
 * 
 * \details
 * Name of the job net
 *
 * @param jobnet : 
 */
//    public void setJobNet (SOSOptionString p_jobnet) { 
//        this.JobNet = p_jobnet;
//    }

                        

/**
 * \var maxJobNetNodes : 
 * maximun number of nodes in one job net.
 *
 */
    @JSOptionDefinition(name = "MaxJobNetNodes", 
    description = "", 
    key = "MaxJobNetNodes", 
    type = "SOSOptionInteger", 
    mandatory = false)
    
    public SOSOptionInteger MaxJobNetNodes = new SOSOptionInteger(this, conClassName + ".MaxJobNetNodes", // HashMap-Key
                                                                "", // Titel
                                                                "500", // InitValue
                                                                "500", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getmaxJobNetNodes : 
 * 
 * \details
 * maximun number of nodes in one job net.
 *
 * \return 
 *
 */
    public SOSOptionInteger  getMaxJobNetNodes() {
        return MaxJobNetNodes;
    }

/**
 * \brief setmaxJobNetNodes : 
 * 
 * \details
 * maximun number of nodes in one job net.
 *
 * @param maxJobNetNodes : 
 */
    public void setMaxJobNetNodes (SOSOptionInteger p_maxJobNetNodes) { 
        this.MaxJobNetNodes = p_maxJobNetNodes;
    }

/**
 * \var CheckJobNetTexture 
 * Additional to the order parameter successor a second parameter predecessor is expected. It defines
 * the relationship to all predecessors of a node in the same manner as in case of the successors.
 * This parameters gives the ability to check the texture of the jobnet. Each succesor defined at
 * one node muest have a predecessor vice versa.
 */
    @JSOptionDefinition(name = "CheckJobNetTexture", 
    description = "", 
    key = "CheckJobNetTexture", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean CheckJobNetTexture = new SOSOptionBoolean(this, conClassName + ".CheckJobNetTexture", // HashMap-Key
                                                                "", // Titel
                                                                "true", // InitValue
                                                                "true", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getCheckJobNetTexture : 
 * \return 
 */
    public SOSOptionBoolean  getCheckJobNetTexture() {
        return CheckJobNetTexture;
    }

/**
 * \var StartOrder
 * Flag to determine if an order should create and start after buildung the jobnet. 
 */
    @JSOptionDefinition(name = "StartOrder", 
    description = "", 
    key = "StartOrder", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean StartOrder = new SOSOptionBoolean(this, conClassName + ".StartOrder", // HashMap-Key
                                                                "", // Titel
                                                                "true", // InitValue
                                                                "true", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getStartOrder : 
 * \return 
 */
    public SOSOptionBoolean  getStartOrder() {
        return StartOrder;
    }

/**
 * \brief setStartOrder : 
 * @param boolean : 
 */
    public void setStartOrder (SOSOptionBoolean p_StartOrder) { 
        this.StartOrder = p_StartOrder;
    }

/**
 * \var CheckFrequency
 * Flag to determine if an order should create and start after buildung the jobnet. 
 */
    @JSOptionDefinition(name = "CheckFrequency", 
    description = "", 
    key = "CheckFrequency", 
    type = "SOSOptionBoolean", 
    mandatory = false)
    
    public SOSOptionBoolean CheckFrequency = new SOSOptionBoolean(this, conClassName + ".CheckFrequency", // HashMap-Key
                                                                "", // Titel
                                                                "true", // InitValue
                                                                "true", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief getCheckFrequency : 
 * \return 
 */
    public SOSOptionBoolean  getCheckFrequency() {
        return CheckFrequency;
    }

/**
 * \brief setCheckFrequency : 
 * @param boolean : 
 */
    public void setCheckFrequency (SOSOptionBoolean p_CheckFrequency) { 
        this.CheckFrequency = p_CheckFrequency;
    }
                        

/**
 * \var port : 
 * 
 *
 */
    @JSOptionDefinition(name = "SCHEDULER_TCP_PORT", 
    description = "", 
    key = "SCHEDULER_TCP_PORT", 
    type = "SOSOptionPortNumber", 
    mandatory = true)
    
    public SOSOptionPortNumber SCHEDULER_TCP_PORT = new SOSOptionPortNumber(this, conClassName + ".SCHEDULER_TCP_PORT", // HashMap-Key
                                                                "", // Titel
                                                                "0", // InitValue
                                                                "0", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getport : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionPortNumber  getSCHEDULER_TCP_PORT() {
        return SCHEDULER_TCP_PORT;
    }

/**
 * \brief setport : 
 * 
 * \details
 * 
 *
 * @param port : 
 */
    public void setSCHEDULER_TCP_PORT (SOSOptionPortNumber p_port) { 
        this.SCHEDULER_TCP_PORT = p_port;
    }

                        

/**
 * \var SCHEDULER_CONFIGURATION_DIRECTORY : 
 * root folder of the job net defintions.
 *
 */
    @JSOptionDefinition(name = "SCHEDULER_CONFIGURATION_DIRECTORY", 
    description = "", 
    key = "SCHEDULER_CONFIGURATION_DIRECTORY", 
    type = "SOSOptionFolderName", 
    mandatory = false)
    
    public SOSOptionFolderName SCHEDULER_CONFIGURATION_DIRECTORY = new SOSOptionFolderName(this, conClassName + ".SCHEDULER_CONFIGURATION_DIRECTORY", // HashMap-Key
                                                                "", // Titel
                                                                " ", // InitValue
                                                                " ", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getSCHEDULER_CONFIGURATION_DIRECTORY : 
 * 
 * \details
 * root folder of the job net defintions.
 *
 * \return 
 *
 */
    public SOSOptionFolderName  getSCHEDULER_CONFIGURATION_DIRECTORY() {
        return SCHEDULER_CONFIGURATION_DIRECTORY;
    }

/**
 * \brief setLiveFolder : 
 * 
 * \details
 * root folder of the job net defintions.
 *
 * @param liveFolder : 
 */
    public void setSCHEDULER_CONFIGURATION_DIRECTORY (SOSOptionFolderName p_liveFolder) { 
        this.SCHEDULER_CONFIGURATION_DIRECTORY = p_liveFolder;
    }


/**
 * \var schedulerId : 
 * scheduler id.
 *
 */
    @JSOptionDefinition(name = "SCHEDULER_ID", 
    description = "", 
    key = "SCHEDULER_ID", 
    type = "SOSOptionString", 
    mandatory = false)
    
    public SOSOptionString SCHEDULER_ID = new SOSOptionString(this, conClassName + ".SCHEDULER_ID", // HashMap-Key
                                                                "", // Titel
                                                                " ", // InitValue
                                                                " ", // DefaultValue
                                                                true // isMandatory
                    );

/**
 * \brief getSCHEDULER_ID : 
 * 
 * \details
 * scheduler id.
 *
 * \return 
 *
 */
    public SOSOptionString  getSCHEDULER_ID() {
        return SCHEDULER_ID;
    }

/**
 * \brief setSCHEDULER_ID : 
 * 
 * \details
 * scheduler id.
 *
 * @param schedulerId : 
 */
    public void setSCHEDULER_ID (SOSOptionString p_schedulerId) { 
        this.SCHEDULER_ID = p_schedulerId;
    }

                        

/**
 * \var time_horizon : 
 * 
 *
 */
    @JSOptionDefinition(name = "time_horizon", 
    description = "", 
    key = "time_horizon", 
    type = "SOSOptionTimeHorizon", 
    mandatory = false)
    
    public SOSOptionTimeHorizon time_horizon = new SOSOptionTimeHorizon(this, conClassName + ".time_horizon", // HashMap-Key
                                                                "", // Titel
                                                                "1:00:00:00", // InitValue
                                                                "1:00:00:00", // DefaultValue
                                                                false // isMandatory
                    );

/**
 * \brief gettime_horizon : 
 * 
 * \details
 * 
 *
 * \return 
 *
 */
    public SOSOptionTimeHorizon  gettime_horizon() {
        return time_horizon;
    }

/**
 * \brief settime_horizon : 
 * 
 * \details
 * 
 *
 * @param time_horizon : 
 */
    public void settime_horizon (SOSOptionTimeHorizon p_time_horizon) { 
        this.time_horizon = p_time_horizon;
    }

                        
        
        
	public JobNetCreatorOptionsSuperClass() {
		objParentClass = this.getClass();
	} // public JobNetCreatorOptionsSuperClass

	public JobNetCreatorOptionsSuperClass(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public JobNetCreatorOptionsSuperClass

		//

	public JobNetCreatorOptionsSuperClass (HashMap <String, String> JSSettings) throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public JobNetCreatorOptionsSuperClass (HashMap JSSettings)

/**
 * \brief setAllOptions - bernimmt die OptionenWerte aus der HashMap
 *
 * \details In der als Parameter anzugebenden HashMap sind Schlssel (Name)
 * und Wert der jeweiligen Option als Paar angegeben. Ein Beispiel fr den
 * Aufbau einer solchen HashMap findet sich in der Beschreibung dieser
 * Klasse (\ref TestData "setJobSchedulerSSHJobOptions"). In dieser Routine
 * werden die Schlssel analysiert und, falls gefunden, werden die
 * dazugehrigen Werte den Properties dieser Klasse zugewiesen.
 *
 * Nicht bekannte Schlssel werden ignoriert.
 *
 * \see JSOptionsClass::getItem
 *
 * @param pobjJSSettings
 * @throws Exception
 */
	public void setAllOptions(HashMap <String, String> pobjJSSettings) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

/**
 * \brief CheckMandatory - prft alle Muss-Optionen auf Werte
 *
 * \details
 * @throws Exception
 *
 * @throws Exception
 * - wird ausgelst, wenn eine mandatory-Option keinen Wert hat
 */
		@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
		, Exception {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
		} // public void CheckMandatory ()

/**
 *
 * \brief CommandLineArgs - bernehmen der Options/Settings aus der
 * Kommandozeile
 *
 * \details Die in der Kommandozeile beim Starten der Applikation
 * angegebenen Parameter werden hier in die HashMap bertragen und danach
 * den Optionen als Wert zugewiesen.
 *
 * \return void
 *
 * @param pstrArgs
 * @throws Exception
 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) throws Exception {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class JobNetCreatorOptionsSuperClass
