/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.history;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import com.sos.hibernate.interfaces.ISOSHibernateDataProvider;
import com.sos.scheduler.history.classes.SchedulerHistoryTableItem;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBItem;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBLayer;

/**
* \class SchedulerOrderHistoryDataProvider 
* 
* \brief SchedulerOrderHistoryDataProvider - 
* 
* \details
*
* \section SchedulerOrderHistoryDataProvider Introduction
*
* \section SchedulerOrderHistoryDataProvider.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author Uwe Risse
* \version 19.01.2012
* \see reference
*
* Created on 19.01.2012 09:31:01
 */

public class SchedulerOrderHistoryDataProvider implements ISOSHibernateDataProvider {

	@SuppressWarnings("unused")
	private final String						conClassName						= "SchedulerOrderHistoryDataProvider";

    private List<SchedulerOrderHistoryDBItem>   listOfSchedulerOrderHistoryDBItems  = null;
//    private List<SchedulerOrderHistoryDBItem>   listOfSchedulerOrderHistoryDBSchedulersItems  = null;
	private SchedulerOrderHistoryDBLayer		schedulerOrderHistoryDBLayer		= null;
	private static Logger						logger								= Logger.getLogger(SchedulerOrderHistoryDataProvider.class);
    private ArrayList							ignoreList=null;

	public SchedulerOrderHistoryDataProvider(File configurationFile) {
		this.schedulerOrderHistoryDBLayer = new SchedulerOrderHistoryDBLayer(configurationFile);
		ignoreList = new ArrayList();
		 

	}

	public SchedulerOrderHistoryFilter getFilter() {
		return schedulerOrderHistoryDBLayer.getFilter();
	}

	public void resetFilter() {
		schedulerOrderHistoryDBLayer.resetFilter();
	}

	public void getData() {
		listOfSchedulerOrderHistoryDBItems = schedulerOrderHistoryDBLayer.getSchedulerOrderHistoryListFromTo();
	}

	public void fillSchedulerIds(CCombo cbSchedulerId) {
		if (listOfSchedulerOrderHistoryDBItems != null) {
		    //Es ist schneller, die vorhandenen Stze zu verwenden.
		    //listOfSchedulerOrderHistoryDBSchedulersItems = schedulerOrderHistoryDBLayer.getSchedulerOrderHistoryListSchedulersFromTo();
			Iterator<SchedulerOrderHistoryDBItem> schedulerOrderHistoryEntries = listOfSchedulerOrderHistoryDBItems.iterator();
			while (schedulerOrderHistoryEntries.hasNext()) {
				SchedulerOrderHistoryDBItem h = schedulerOrderHistoryEntries.next();
				if (cbSchedulerId.indexOf(h.getSpoolerId()) < 0) {
					logger.debug("... cbSchedulerId --> : " + h.getSpoolerId());
					cbSchedulerId.add(h.getSpoolerId());
				}
			}
		}
	}

	public String getLogAsString(Table tableSchedulerOrderHistory) {
		String log = "";
		if (tableSchedulerOrderHistory.getSelectionIndex() >= 0) {
			TableItem t = tableSchedulerOrderHistory.getItem(tableSchedulerOrderHistory.getSelectionIndex());
			SchedulerOrderHistoryDBItem h = (SchedulerOrderHistoryDBItem) t.getData();
			if (h.getHistoryId() != null) {
				log = getLogAsString(h.getHistoryId());
			}
		}
		return log;
	}

	public String getLogAsString(Long id) {
		String log = "";
		try {
			SchedulerOrderHistoryDBItem schedulerOrderHistoryDBItem = schedulerOrderHistoryDBLayer.get(id);
			if (schedulerOrderHistoryDBItem != null && schedulerOrderHistoryDBItem.getLog() != null) {
				log = schedulerOrderHistoryDBItem.getLogAsString();
			}
		}
		catch (IOException e1) {
			e1.printStackTrace();
		}
		return log;
	}

	public void fillTableShort(Table table) {
		if (listOfSchedulerOrderHistoryDBItems != null) {

			Iterator schedulerOrderHistoryEntries = listOfSchedulerOrderHistoryDBItems.iterator();
			while (schedulerOrderHistoryEntries.hasNext()) {
				SchedulerOrderHistoryDBItem h = (SchedulerOrderHistoryDBItem) schedulerOrderHistoryEntries.next();
				if (schedulerOrderHistoryDBLayer.getFilter().isFiltered(h)) {
				}
				else {
					final SchedulerHistoryTableItem newItemTableItem = new SchedulerHistoryTableItem(table, SWT.BORDER);
					newItemTableItem.setDBItem(h);

					logger.debug("...creating tableItem: " + h.getJobChain() + "/" + h.getOrderId() + ":" + table.getItemCount());
					newItemTableItem.setData(h);
					newItemTableItem.setColor();
					newItemTableItem.setColumnsShort();
				}
			}
		}
	}

	public void fillTable(Table table) {

		if (listOfSchedulerOrderHistoryDBItems != null) {
			Iterator schedulerOrderHistoryEntries = listOfSchedulerOrderHistoryDBItems.iterator();
			while (schedulerOrderHistoryEntries.hasNext()) {
				SchedulerOrderHistoryDBItem h = (SchedulerOrderHistoryDBItem) schedulerOrderHistoryEntries.next();
				if (schedulerOrderHistoryDBLayer.getFilter().isFiltered(h)) {
				}
				else {
					final SchedulerHistoryTableItem newItemTableItem = new SchedulerHistoryTableItem(table, SWT.BORDER);
					newItemTableItem.setDBItem(h);

					logger.debug("...creating tableItem: " + h.getJobChain() + "/" + h.getOrderId() + ":" + table.getItemCount());
					newItemTableItem.setData(h);
					newItemTableItem.setColor();
					newItemTableItem.setColumns();
				}
			}
		}
	}

	@Override
	public void commit() {
		schedulerOrderHistoryDBLayer.commit();		
	}
	
	public void addToIgnorelist(SchedulerOrderHistoryDBItem h){
		schedulerOrderHistoryDBLayer.getFilter().getIgnoreList().add(h);
	}

	
	public void resetIgnoreList(){
		schedulerOrderHistoryDBLayer.getFilter().getIgnoreList().reset();
	}	 

}
