/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// sosstrg0.h

#ifndef __SOSSTRG0_H
#define __SOSSTRG0_H

#if !defined __SYSDEP_H
#   include "sysdep.h"
#endif

namespace sos
{

typedef unsigned int uint;

#if !defined __STRING_H  &&  !defined _STRING_H
#   include <string.h>
#endif

inline uint             length      ( const char* s )   { return strlen( s ); }
inline const char*      c_str       ( const char* );    // in sos.h implementiert
       Big_int          as_big_int  ( const char* );
       Ubig_int         as_ubig_int ( const char* );
       long             as_long     ( const char* );
       unsigned long    as_ulong    ( const char* );
       Bool             as_bool     ( const char* );
       Bool             as_bool     ( const char*, Bool deflt );
       int              as_int      ( const char* );
       int              as_int      ( const char*, int deflt );
       uint             as_uint     ( const char* );
       short            as_short    ( const char* );
       unsigned short   as_ushort   ( const char* );

#ifdef SYSTEM_INT64
       int64            as_int64    ( const char* );
       uint64           as_uint64   ( const char* );
#endif
       int32            as_int32    ( const char* );
       uint32           as_uint32   ( const char* );
       int16            as_int16    ( const char* );
       uint16           as_uint16   ( const char* );
       int              as_uintK    ( const char* );    // nK: *1024, nM: *1024*1024

inline int4             as_int4     ( const char* text )     { return as_int32( text ); }
inline uint4            as_uint4    ( const char* text )     { return as_uint32( text ); }
inline int2             as_int2     ( const char* text )     { return as_int16( text ); }

inline uint2            as_uint2    ( const char* text )     { return as_uint16( text ); }


       double     c_str_as_double   ( const char* );
inline double           as_double   ( const char* s )   { return c_str_as_double( s ); }
       Bool            _empty       ( const char* );
inline Bool             empty       ( const char* );
inline uint             position    ( const char*, char to_find, unsigned int pos = 0 );
       uint             position    ( const char*, const char* to_find, unsigned int pos = 0 );
       uint             length_without_trailing_char( const char* , uint len, char );
inline uint             length_without_trailing_spaces( const char* ptr, uint len )             { return length_without_trailing_char( ptr, len, ' ' ); }
inline uint             length_without_trailing_spaces( const char* str )                       { return length_without_trailing_spaces( str, strlen( str ) ); }

       unsigned int     position_in_string0( const char* str, char to_find, unsigned int pos );

void                   _ltrim       ( char* );
inline void             ltrim       ( char* str )                                               { if( str[ 0 ] <= ' ' )  _ltrim( str ); }
void                    rtrim       ( char* );
void                    trim        ( char* );

//=======================================================================================INLINE

//-------------------------------------------------------------------------------------position

inline unsigned int position( const char* str, char to_find, unsigned int pos )
{
    return position_in_string0( str, to_find, pos );
}

//----------------------------------------------------------------------------------------empty

inline Bool empty( const char* s )
{
#   if !defined SOS_OPTIMIZE_SIZE
        if( !s           )  return 1;
        if( s[0] == '\0' )  return 1;
#   endif
    return _empty( s );
}


} //namespace sos

#endif
