/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: spooler_module_process.h 13807 2009-11-18 08:59:21Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#ifndef __SPOOLER_MODULE_PROCESS_H
#define __SPOOLER_MODULE_PROCESS_H

namespace sos {
namespace scheduler {

//--------------------------------------------------------------------------Process_module_instance

struct Process_module_instance : Module_instance
{
#   ifndef Z_WINDOWS
        struct Process_event : Event
        {
                                        Process_event               ( const string& name )                  : Event(name) {}

            virtual void                close                       ();
            bool                        signaled                    ();
            virtual bool                wait                        ( double seconds = INT_MAX );
                                        operator bool               ()                                      { return _pid != 0; }

            int                        _pid;
            int                        _process_signaled;
            int                        _process_exit_code;
        };
#   endif


                                Process_module_instance     ( Module* );
                               ~Process_module_instance     ();

  //virtual void                attach_task                 ( Task*, Prefix_log* );

    void                        init                        ();
    bool                        load                        ();
    void                        start                       ();
    void                        close__end                  ();
    void                        close_process               ();
    Async_operation*            begin__start                ();
    bool                        begin__end                  ();
    void                        end__end                    ();
    Variant                     step__end                   ();
    Variant                     call                        ( const string& name );
    Variant                     call                        ( const string& name, const Variant& param, const Variant& );
    bool                        name_exists                 ( const string& name );
    bool                        loaded                      ()                                      { return _idispatch != NULL; }
    bool                        callable                    ()                                      { return _idispatch != NULL; }
    bool                        kill                        ();
    bool                        process_has_signaled        ();
    string                      get_first_line_as_state_text();
    void                        fill_process_environment_with_params();
    string                      step_result                 ();
    void                        transfer_back_order_params  ();
    void                        fetch_parameters_from_process( Com_variable_set* );
    File_path                   stdout_path                 ()                                      { return _stdout_file.path(); }
    File_path                   stderr_path                 ()                                      { return _stderr_file.path(); }
    bool                        try_delete_files            ( Has_log* );
    std::list<File_path>        undeleted_files             ();
    bool                        is_kill_thread_running      ()                                      { return _kill_thread  &&  _kill_thread->thread_is_running(); }

  private:
    void                        close_handle                ();
    string                      program_path                ();
    std::vector<string>         get_string_args             ();
    string                      get_parameter_name          (const string suffix);


    Fill_zero                  _zero_;
    File                       _stdout_file;
    File                       _stderr_file;
  //bool                       _stdout_logged;
    Mapped_file                _order_params_file;

    bool                       _is_killed;
    File                       _shell_file;
    string                     _process_param;

#   ifdef Z_WINDOWS
        //Process_id             _pid;
        Event                  _process_handle;
#    else
        Process_event          _process_handle;
        int                    _pid_to_unregister;
#   endif

    ptr<Thread>                _kill_thread;
    bool                       _spooler_process_before_called;
};

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

#endif
